@@ -1,5 +1,6 @@
 import pyutilib.workflow
 import os.path
+import os
 currdir = os.path.dirname(os.path.abspath(__file__))+os.sep
 
 import sys
@@ -31,3 +32,6 @@ w = pyutilib.workflow.Workflow()
 w.add(H)
 print w(dir=currdir+'dummy')
 # @:ex
+
+if os.path.exists(currdir+'logfile'):
+    os.remove(currdir+'logfile')
