@@ -38,24 +38,22 @@ class TaskDriver(object):
         self.parser.print_help()        #pragma:nocover
 
     def parse_args(self, args=None):
+        #
+        # Call the parser to parse the arguments
+        #
         if args is None:
+            # sys.argv arguments
             ret = self.parser.parse_args()
         else:
+            # args arguments
             ret = self.parser.parse_args(args)
+        #
+        # Set options, based on the parser return value.
+        # The parser options are passed into the workflow.
+        #
         opt = Options()
         for key in dir(ret):
             if key[0] != '_' and key != 'subparser_name':
                 opt[key] = getattr(ret,key)
         return self.wf[ ret.subparser_name ](opt)
 
-    def process_core_arguments(self, opt):
-        if opt.help_commands:
-            cmds = sorted(self.wf.keys())
-            if len(cmds) > 0:
-                print 'Available subcommands:'
-                for cmd in cmds:
-                    print '   %s' % cmd
-            else:
-                print 'No subcommands are registered'
-            print ''
-        return Options()
