@@ -32,7 +32,7 @@ def define_connection(cls, from_port, to_port):
 class Port(object):
     """A class that represents an input or output port on a task."""
 
-    def __init__(self, name, task, optional=False, value=None, action=None, constant=False):
+    def __init__(self, name, task, optional=False, value=None, action=None, constant=False, default=None, doc=None):
         """Constructor."""
         self.name=name
         self.task=task
@@ -46,6 +46,8 @@ class Port(object):
         self.output_connections = []
         self.set_value(value)
         self._ready = False
+        self.default=default
+        self.doc=doc
 
     def reset(self):
         self._ready = False
@@ -61,6 +63,8 @@ class Port(object):
 
     def get_value(self):
         """Get the value of this port."""
+        if self.value is None:
+            return self.default
         return self.value
 
     def set_value(self, value):
@@ -108,15 +112,15 @@ class Port(object):
 
     def validate(self):
         if self.action in ['store', 'store_any']:
-            if not self.optional and self.value is None:
+            if not self.optional and self.get_value() is None:
                 raise ValueError, "Task %s Port %s requires a nontrivial value.  Value specified is None." % (str(self.task.id), self.name)
         #
         elif self.action in ['append', 'append_any']:
-            if not self.optional and self.value is None:
+            if not self.optional and self.get_value() is None:
                 raise ValueError, "Task %s Port %s requires a nontrivial value.  All input connections have value None." % (str(self.task.id), self.name)
         #
         elif self.action in ['map', 'map_any']:
-            if not self.optional and self.value is None:
+            if not self.optional and self.get_value() is None:
                 raise ValueError, "Task %s Port %s requires a nontrivial value.  All input connections have value None." % (str(self.task.id), self.name)
 
     def _repn_(self):
@@ -145,7 +149,7 @@ class Port(object):
         return pprint.pformat(self._repn_(), 2) #pragma:nocover
 
     def ready(self):
-        if not self.value is None:
+        if not self.get_value() is None:
             return True
         if len(self.input_connections) == 0:
             return self._ready
@@ -184,9 +188,9 @@ class Ports(dict):
         """Set the name of this class instance."""
         self._name_ = name
 
-    def declare(self, name, optional=False, action=None, constant=False):
+    def declare(self, name, optional=False, action=None, constant=False, default=None, doc=None):
         """Declare a port."""
-        port = Port(name, self._task, optional=optional, action=action, constant=constant)
+        port = Port(name, self._task, optional=optional, action=action, constant=constant, default=default, doc=doc)
         setattr(self, name, port)
         return port
 
