@@ -16,6 +16,7 @@ class Resource(object):
 
     def __init__(self, id=None, name=None):
         if not id is None:
+            # TBD:  does this ever happen?
             self.id = id
         else:
             self.id = globals.unique_id()
@@ -23,22 +24,23 @@ class Resource(object):
             self.description = "Resource"+str(self.id)
             self.name = self.description
         else:
+            # TBD:  does this ever happen?
             self.description = name
             self.name = name
         self._busy=False
 
     def available(self):
-        return not self._busy
+        return not self._busy       #pragma:nocover
 
     def lock(self):
-        self._busy=False
+        self._busy=True             #pragma:nocover
 
     def unlock(self):
-        self._busy=False
+        self._busy=False            #pragma:nocover
 
     def __repr__(self):
-        return str(self)
+        return str(self)            #pragma:nocover
 
     def __str__(self):
-        return "%s" % str(self.name)
+        return "%s" % str(self.name)#pragma:nocover
 
