@@ -44,6 +44,7 @@ class Task(object):
         self.input_controls.set_name(self.name+'-input-controls')
         self.output_controls = port.OutputPorts(self)
         self.output_controls.set_name(self.name+'-output-controls')
+        self.debug = False
 
     def add_resource(self, resource):
         """Add a resource that is required for this task to execute."""
@@ -73,7 +74,7 @@ class Task(object):
         """Return the set of ids for tasks that precede this task in the workflow."""
         return set(task.id for task in self.prev_tasks())
            
-    def execute(self):
+    def execute(self, debug=False):
         """Execute this task."""
         raise ValueError, "There is no default execution for an abstract Task object! Task=%s" % str(self) #pragma:nocover
 
@@ -241,7 +242,7 @@ class Component(Task):
 
     def __init__(self, *args, **kwds):
         """Constructor."""
-        Task.__init__(self, *args, **kwds)
+        Task.__init__(self, *args, **kwds)          #pragma:nocover
 
 
 class EmptyTask(Task):
