@@ -13,6 +13,16 @@ is_python24or25 = False
 if sys.version_info[0:2] < (2, 6):
     is_python24or25 = True
 
+
+class DummyResource(pyutilib.workflow.Resource):
+
+    def __init__(self, name=None):
+        pyutilib.workflow.Resource.__init__(self)
+
+    def available(self):
+        return False
+
+
 class TaskA(pyutilib.workflow.Task):
 
     def __init__(self, *args, **kwds):
@@ -47,6 +57,17 @@ class TaskAA2(pyutilib.workflow.Task):
         self.z = sum(self.x.keys())
 
 
+class TaskAA3(pyutilib.workflow.Task):
+
+    def __init__(self, *args, **kwds):
+        pyutilib.workflow.Task.__init__(self, *args, **kwds)
+        self.inputs.declare('x', action='map_any')
+        self.outputs.declare('z')
+
+    def execute(self):
+        self.z = sum(self.x.keys())
+
+
 class TaskB(pyutilib.workflow.Task):
 
     def __init__(self, *args, **kwds):
@@ -228,6 +249,7 @@ Task7 prev: [1] next: [] resources: []""")
                      'Action': 'store',
                      'Connections': { 'Inputs': [ 'DirectConnector: from=(2) to=(1) False'],
                                       'Outputs': []},
+                     'Constant': 'False',
                      'Name': 'x',
                      'Optional': 'False',
                      'Ready': 'False',
@@ -236,6 +258,7 @@ Task7 prev: [1] next: [] resources: []""")
               'y': { 'A_TYPE': 'Port',
                      'Action': 'store',
                      'Connections': { 'Inputs': [], 'Outputs': []},
+                     'Constant': 'False',
                      'Name': 'y',
                      'Optional': 'False',
                      'Ready': 'False',
@@ -253,6 +276,7 @@ Task7 prev: [1] next: [] resources: []""")
                'z': { 'A_TYPE': 'Port',
                       'Action': 'store',
                       'Connections': { 'Inputs': [], 'Outputs': []},
+                      'Constant': 'False',
                       'Name': 'z',
                       'Optional': 'False',
                       'Ready': 'False',
@@ -269,44 +293,11 @@ Task7 prev: [1] next: [] resources: []""")
         A = TaskA()
         B = TaskB()
         A.inputs.x = B.outputs.b
-        inputs_base = """{ 'A_TYPE': 'Port',
-  'Mode': 'inputs',
-  'Name': 'Task1-inputs',
-  'Owner': 'Task1 prev: [2] next: [] resources: []',
-  'x': { 'A_TYPE': 'Port',
-         'Action': 'store',
-         'Connections': { 'Inputs': [ 'DirectConnector: from=(2) to=(1) False'],
-                          'Outputs': []},
-         'Name': 'x',
-         'Optional': 'False',
-         'Ready': 'False',
-         'Task': '1',
-         'Value': 'None'},
-  'y': { 'A_TYPE': 'Port',
-         'Action': 'store',
-         'Connections': { 'Inputs': [], 'Outputs': []},
-         'Name': 'y',
-         'Optional': 'False',
-         'Ready': 'False',
-         'Task': '1',
-         'Value': 'None'}}"""
-
-        outputs_base = """{ 'A_TYPE': 'Port',
-  'Mode': 'outputs',
-  'Name': 'Task1-outputs',
-  'Owner': 'Task1 prev: [2] next: [] resources: []',
-  'z': { 'A_TYPE': 'Port',
-         'Action': 'store',
-         'Connections': { 'Inputs': [], 'Outputs': []},
-         'Name': 'z',
-         'Optional': 'False',
-         'Ready': 'False',
-         'Task': '1',
-         'Value': 'None'}}"""
-
-        self.assertEqual(str(A.inputs), inputs_base)
-        self.assertEqual(str(A.outputs), outputs_base)
-
+        OUTPUT = open(currdir+'test3.out','w')
+        print >>OUTPUT, str(A.inputs)
+        print >>OUTPUT, str(A.outputs)
+        OUTPUT.close()
+        self.assertFileEqualsBaseline(currdir+'test3.out', currdir+'test3.txt')
 
     @unittest.skipIf( is_python24or25, "There is a slight (space) formatting difference from pformat from Python2.6.  Skipping test.")
     def test4(self):
@@ -316,52 +307,11 @@ Task7 prev: [1] next: [] resources: []""")
         pyutilib.workflow.reset_id_counter()
         A = TaskA()
         A.inputs['x'] = 2
-        base = """{ 'A_TYPE': 'Task',
-  'Id': 1,
-  'InputControls': { 'A_TYPE': 'Port',
-                     'Mode': 'inputs',
-                     'Name': 'Task1-input-controls',
-                     'Owner': 'Task1 prev: [] next: [] resources: []'},
-  'Inputs': { 'A_TYPE': 'Port',
-              'Mode': 'inputs',
-              'Name': 'Task1-inputs',
-              'Owner': 'Task1 prev: [] next: [] resources: []',
-              'x': { 'A_TYPE': 'Port',
-                     'Action': 'store',
-                     'Connections': { 'Inputs': [], 'Outputs': []},
-                     'Name': 'x',
-                     'Optional': 'False',
-                     'Ready': 'True',
-                     'Task': '1',
-                     'Value': '2'},
-              'y': { 'A_TYPE': 'Port',
-                     'Action': 'store',
-                     'Connections': { 'Inputs': [], 'Outputs': []},
-                     'Name': 'y',
-                     'Optional': 'False',
-                     'Ready': 'False',
-                     'Task': '1',
-                     'Value': 'None'}},
-  'Name': 'Task1',
-  'OutputControls': { 'A_TYPE': 'Port',
-                      'Mode': 'outputs',
-                      'Name': 'Task1-output-controls',
-                      'Owner': 'Task1 prev: [] next: [] resources: []'},
-  'Outputs': { 'A_TYPE': 'Port',
-               'Mode': 'outputs',
-               'Name': 'Task1-outputs',
-               'Owner': 'Task1 prev: [] next: [] resources: []',
-               'z': { 'A_TYPE': 'Port',
-                      'Action': 'store',
-                      'Connections': { 'Inputs': [], 'Outputs': []},
-                      'Name': 'z',
-                      'Optional': 'False',
-                      'Ready': 'False',
-                      'Task': '1',
-                      'Value': 'None'}}}"""
-
         self.assertEqual(A.inputs['x'].get_value(), 2)
-        self.assertEqual(repr(A), base)
+        OUTPUT = open(currdir+'test4.out','w')
+        print >>OUTPUT, repr(A)
+        OUTPUT.close()
+        self.assertFileEqualsBaseline(currdir+'test4.out', currdir+'test4.txt')
 
     def test5(self):
         pyutilib.workflow.reset_id_counter()
@@ -378,7 +328,7 @@ Task7 prev: [1] next: [] resources: []""")
         w.set_options(['--i=3','--a=2'])
         try:
             w()
-            self.fail("Expected TypeError because the inputs are not defined")
+            self.fail("Expected ValueError because the inputs are not defined")
         except ValueError:
             pass
 
@@ -504,6 +454,63 @@ Task7 prev: [1] next: [] resources: []""")
         except ValueError:
             pass
 
+    def test_error7(self):
+        A1 = TaskA()
+        A2 = TaskA()
+        A3 = TaskA()
+        A3.inputs.x = A1.outputs.z
+        try:
+            A3.inputs.x = A2.outputs.z
+            self.fail("Expected ValueError because we're connecting two outputs to an input with action 'store'")
+        except:
+            pass
+
+    def test_error8(self):
+        A1 = TaskAA1()
+        A2 = TaskAA1()
+        A2.inputs.x = A1.outputs.z
+        try:
+            A2.inputs.x.validate()
+            self.fail("Expected ValueError because the 'x' input does not have a nontrivial value")
+        except:
+            pass
+        self.assertFalse(A2.ready())
+        
+    def test_error9(self):
+        A1 = TaskAA2()
+        A2 = TaskAA2()
+        A2.inputs.x = A1.outputs.z
+        try:
+            A2.inputs.x.validate()
+            self.fail("Expected ValueError because the 'x' input does not have a nontrivial value")
+        except:
+            pass
+        self.assertFalse(A2.ready())
+        
+    def test_error10(self):
+        A1 = TaskAA3()
+        A2 = TaskAA3()
+        A3 = TaskAA3()
+        A2.inputs.x = A1.outputs.z
+        A2.inputs.x = A3.outputs.z
+        try:
+            A2.inputs.x.validate()
+            self.fail("Expected ValueError because the 'x' input does not have a nontrivial value")
+        except:
+            pass
+        self.assertFalse(A2.ready())
+        A1.outputs.z = 1
+        self.assertTrue(A2.ready())
+
+    def test_error11(self):
+        A1 = TaskA()
+        A1.add_resource(DummyResource())
+        try:
+            A1()
+            self.fail("Expected IOError because resource is busy.")
+        except IOError:
+            pass
+    
     def test_driver1(self):
         driver = pyutilib.workflow.TaskDriver()
         driver.register_task('workflow.selection')
@@ -523,5 +530,16 @@ Task7 prev: [1] next: [] resources: []""")
         except ValueError:
             pass
 
+    def test_driver2(self):
+        driver = pyutilib.workflow.TaskDriver()
+        driver.register_task('workflow.selection')
+        try:
+            # Set sys.argv and then parse arguments
+            sys.argv=['foo','-h']
+            driver.parse_args()
+            self.fail("Expected system exit because of bad options")
+        except SystemExit:
+            pass
+
 if __name__ == "__main__":
     unittest.main()
