@@ -21,6 +21,11 @@ from pyutilib.misc import Options
 import pprint
 from collections import deque
 
+try:
+    from collections import OrderedDict
+except:
+    from ordereddict import OrderedDict
+
 
 def _collect_parser_groups(t):
     for key in t._parser_group:
@@ -54,7 +59,8 @@ class Workflow(Task):
         self.add(self._final_task)
 
     def add(self, task, loadall=True):
-        #print "ADD",task.id
+        if self.debug:
+            print "ADDING",task.id          #pragma:nocover
         if task.id == NoTask.id:
             return
         if task.id in self._tasks:
@@ -72,19 +78,22 @@ class Workflow(Task):
             if len(task.inputs[name].from_tasks()) > 0:
                 continue
 
-            if not name in self._start_task.outputs:
-                self._start_task.outputs.declare(name)
-                #print "Declaring input "+name
-                self.inputs.declare(name, optional=True)
-            #print "Z",self._start_task.outputs
-            #print "Z",self._start_task.outputs[name]
-            # TODO: this is a bit of a hack...
-            val = getattr(task.inputs,name).get_value()
-            try:
-                setattr(task.inputs, name, getattr(self._start_task.outputs, name))
-            except ValueError:
-                pass
-            getattr(self.inputs,name).set_value(val)
+            if not task.inputs[name].constant:
+                #
+                # Constant input values are not added to the start task
+                #
+                if not name in self._start_task.outputs:
+                    self._start_task.outputs.declare(name)
+                    #print "Declaring input "+name
+                    self.inputs.declare(name, optional=True)
+                # TODO: this is a bit of a hack...
+                val = getattr(task.inputs,name).get_value()
+                try:
+                    setattr(task.inputs, name, getattr(self._start_task.outputs, name))
+                except ValueError:
+                    # TBD: when do we get this exception?
+                    pass
+                getattr(self.inputs,name).set_value(val)
             #getattr(self._start_task.outputs,name).set_value(val)
             #print "Z",task.inputs
             #print "Z",self._start_task.outputs[name].from_task.id
@@ -114,11 +123,13 @@ class Workflow(Task):
             if not val is None:
                 self._start_task.outputs[i].set_value( val )
                 self._start_task.outputs[i].set_ready()
-        # Q: is this redundant???
-        for key in kwds:
-            if key not in self._start_task.outputs:
-                raise ValueError, "Cannot specify value for option %s.  Valid option names are %s" % (key, self._start_task.outputs.keys())
-            self._start_task.outputs[key].set_value( kwds[key] )
+        #
+        # TBD: this appears to be redundant
+        #
+        #for key in kwds:
+        #    if key not in self._start_task.outputs:
+        #        raise ValueError, "Cannot specify value for option %s.  Valid option names are %s" % (key, self._start_task.outputs.keys())
+        #    self._start_task.outputs[key].set_value( kwds[key] )
 
     def _call_fini(self, *options, **kwds):
         ans = Options()
@@ -151,46 +162,76 @@ class Workflow(Task):
 
     def execute(self):
         #return self._dfs_([self._start_task.id], lambda t: t.__call__())
-        #print '---------------'
-        #print '---------------'
-        #print repr(self)
-        #print '---------------'
+        if self.debug:                              #pragma:nocover
+            print self.name, '---------------'
+            print self.name, '---------------'
+            print self.name, '   STARTING'
+            print self.name, '---------------'
+            print self.name, '---------------'
+        #
         queued = set([self._start_task.id])
         queue = deque([self._start_task])
-        waiting = {}
-        while len(queue) > 0:
-            task = queue.popleft()
-            #print self.name, "TASK   ",repr(task)
-            ##print self.name, "QUEUE  ",queued
-            ##print self.name, "WAITING",waiting.keys()
-            ##print "Executing Task "+task.name,task.next_task_ids()
-            queued.remove(task.id)
-            task()
+        waiting = OrderedDict()
+        while len(queue)+len(waiting) > 0:
             for id in waiting.keys():
                 t = waiting[id]
                 if not t.id in queued and t.ready():
-                    ##print self.name, "Waiting task",t.name,t.id,t.ready()
-                    #print self.name, "HERE",t,repr(t)
+                    #
+                    if self.debug:                  #pragma:nocover
+                        print self.name, "WAITING: ",t.id," not queued and task ready"
+                        print self.name, "Waiting task",t.name,t.id,t.ready()
+                        #print self.name, "HERE",t,repr(t)
+                    #
                     queue.append(t)
                     queued.add(t.id)
                     del waiting[t.id]
+            if len(queue) == 0:
+                break
+                # TBD: should we sleep and add a timelimit before raising this exception?
+                #if len(waiting) == 0:
+                #    break
+                #print self.name, "ERROR", waiting.keys()
+                #raise RuntimeError, "Workflow failed to terminate normally.  All available tasks are blocked."
+            task = queue.popleft()
+            #
+            if self.debug:                          #pragma:nocover
+                print self.name, "TASK   ",str(task)
+                print self.name, "QUEUE  ",queued
+                print self.name, "WAITING",waiting.keys()
+                print self.name, "Executing Task "+task.name,task.next_task_ids()
+            #
+            queued.remove(task.id)
+            task()
             for t in task.next_tasks():
                 if t.id in queued:
                     continue
                 if t.ready():
-                    ##print self.name, "Scheduling task",t.name,t.id,t.ready()
-                    #print self.name, "HERE",t,repr(t)
+                    #
+                    if self.debug:                  #pragma:nocover
+                        print self.name, "NEXT: ",t.id," not queued and task ready"
+                        print self.name, "Scheduling task",t.name,t.id,t.ready()
+                        #print self.name, "HERE",t,repr(t)
+                    #
                     queue.append(t)
                     queued.add(t.id)
                     if t.id in waiting:
                         del waiting[t.id]
                 else:
-                    waiting[t.id] = t
-                    ##print self.name, "Ignoring task",t.name, t.id,t.ready()
-                    #print self.name, "HERE",t,repr(t)
-            #print self.name, "QUEUE  ",queue
-            #print self.name, "WAITING",waiting.keys()
-        #print '---------------'
+                    if not t.id in waiting:
+                        waiting[t.id] = t
+                    #
+                    if self.debug:                  #pragma:nocover
+                        print self.name, "NEXT: ",t.id," not queued and task NOT ready"
+                        print self.name, "Ignoring task",t.name, t.id,t.ready()
+                        #print self.name, "HERE",t,repr(t)
+            if self.debug:                          #pragma:nocover
+                print self.name, "FINAL QUEUE  ",queued
+                print self.name, "FINAL WAITING",waiting.keys()
+                print self.name, '---------------'
+                print self.name, "    LOOP"
+                print self.name, '---------------'
+        if self.debug:                              #pragma:nocover
+            print self.name, '---------------'
 
     def __str__(self):
         return "\n".join(["Workflow %s:" % self.name]+self._dfs_([self._start_task.id], lambda t: str(t)))
